<?php
/*

Google Map API for PmWiki
=========================

Copyright Statement
-------------------

Copyright (c) 2006, Benjamin C. Wilson. All Rights Reserved. You have
permission to use this software for web site use provided this copyright
statement remains intact. You may not otherwise republish or alter this software
without prior permission of the author. License to publish granted
PmWiki.org for distribution to the PmWiki community. This copyright
statement must accompany the script.

You may contact the author at ameen@dausha.net. This software is
maintained at http://www.dausha.net/GoogleMapAPI/.

Introduction
------------

The Google Map API is a simple markup way to add map support to
your PmWiki site. It complies with the Google Map API v2.0 released
in April, 2006.

Installation
------------

   1. First, you must have a Google Map API certificate. To obtain one,
      visit: http://www.google.com/apis/maps/signup.html. One is needed
      for each web site.
   2. In the configuration file (e.g. local/config.php), set the
      certificate: $GmapCert = '/your gmap cert/';
   3. After any other $Gmap-variables, include the script:
      include_once('/path/to/gmap_api-1.3.0.php');
   4. In your site template, add the $GmapJS to the end of the page
      to avoid javascript problems:
      		$GmapJS</body></html>
      And if you expect to use the polyline feature, you must expand the
      <html> to:
      		<html xmlns="http://www.w3.org/1999/xhtml" xmlns:v="urn:schemas-microsoft-com:vml">

Usage
------

At a minimum, you must use "(:gmap:)," but to set the latitude and
longitude, you will have to use a free geocoder (e.g. http://geocoder.us/).
The examples below show how you can add text to a bubble over the
marker and destination address that allows the site visitor to find
directions to the location.

Examples:
~~~~~~~~~

(:gmap lat=38.897639 lon=-77.036583:) "Minimum for pointing to a location."

(:gmap lat=38.897639 lon=-77.036583 daddr="1600 Pennsylvania, 22303"
msg="Joe's Bar and Grill 1600 Ranch Drive Washington, DC 22223":)
"Full example with the destination address and message."

(:gmapoint lat=39.01 lon=-77.1:)

Polylines:
	(:gmapolyline id=1 lat=39.01 lon=-77.1 [options]:)
	To start new polyline use one or more of the options: 'color',
    'thick' or 'opacy'.
	For additional points on polyline use only 'lat' and 'lon'.

Variables
---------

These are the variables that influence the behavior of the markup. The
defaults are recommended for use in the configuration file for your
PmWiki Installation. The markup options may be added by the author.

Defaults
~~~~~~~~

    * $EnableGmapOverviewControl = enables the Google API overview control
    * $EnableGmapTypeControl = enables the Google API type control
    * $EnableGmapCenterAsPoint = enables the center coordinate as point
    * $EnableGmapCalculatedCenter = enables calculated center coordinate
    * $GmapCtrl = default control size (small|large) defaults to large
    * $GmapLon = default longitude
    * $GmapLat = default latitude
    * $GmapZom = default zoom
    * $GmapJS (leave blank)
    * $GmapType = default map type (normal, satellite, hybrid).
    * $GmapPLColor = default color for polylines.
    * $GmapPLThick = default thickness for polylines.
    * $GmapPLOpacy = default opacity for polylines.

Markup Options
~~~~~~~~~~~~~~

    * ctrl = designates the size of the arrow button control (small|large)
    * daddr = destination address (which allows directions to the
      address)
    * lat = lattitude
    * lon = longitude
    * cpoint = use center coordinate as point, accepts 'y' or 'n' accordingly.
    * ccalc = use calculated center coordinate as map center, accepts 'y'
      or 'n' accordingly.
    * oview = toggles from site default, accepts 'y' or 'n' accordingly.
    * msg = what text is displayed in the window over the marker.
      You will have to provide the HTML.
    * mtype = toggles from site default, accepts 'y' or 'n' accordingly.
    * type = the map type ('nor'mal, 'hyb'rid, 'sat'ellite).
    * zom = the zoom level.
    For use in polylines:
    * color = color of polyline in format '#000000' as in normal html.
    * thick = thickness of polyline in pixels.
    * opacy = opacity of polyline, anything between 0.0 (invisible) to 1.0 (full).

Release Notes
-------------

    * v1.0 - May 17, 2006: Initial Public Release by Ben Wilson.
    * v1.0.1 - May 22, 2006: Release by Ben Wilson.
       * Incorporated Google Map API v2.0+ compliance corrections by Des.
       * Replace $GmapCert with $HTMLHeaderFmt call.
       * Fixed problem with CSS via $HTMLStylesFmt.
       * Added support for dynamic scaling of controller.
       * Added the overview map.
       * Added support for map type (satellite, normal or hybrid).
       * Only add the directions block when directions are offered.
         Before, directions were only offered when a message was given.
       * Only add the message block when the message is offered.
    * v1.0.2 - May 22, 2006 (pm): Des Corrections
       * Added map type control buttons
       * Fixed default operation of map control sizing
       * Fixed spelling error that prevented the parser selecting the
	     desired map control size.
    * v1.0.3 - May 23, 2006: Ben Wilson
       * Added options for height-width of map.
    * v1.1.0 - May 26, 2006: Ben Wilson
       * Added multiple point capability.
       * Added ability to toggle map overview.
       * Added ability to toggle map type control.
    * v1.2.0 - June 3, 2006: Ben Wilson
    * v1.2.1 - August 4, 2006: Helge Larsen
       * Added support for locales using decimalseparator other than "." as in Denmark
       * Added ability to toggle if the center coordinate is added as point
       * Added ability to toggle if map center coordinate is calculated
    * v1.3.0 - August 7, 2006: Ben Wilson, Helge Larsen
       * Added support for multiple polylines

*/

$MapHtml = "<div id='map'></div>";
SDV($EnableGmapTypeControl, 1);
SDV($EnableGmapOverviewControl, 1);
SDV($EnableGmapCenterAsPoint, 1);
SDV($GmapCtrl, 'large');
SDV($GmapHeight, '400px');
SDV($GmapLon,  '-122.141944');
SDV($GmapLat,  '37.441944');
SDV($GmapType, 'nor');
SDV($GmapWidth, '100%');
SDV($GmapZom,  8);
SDV($GmapZIPFile, '');
SDV($GmapPLColor, '#880088');
SDV($GmapPLThick, 3);
SDV($GmapPLOpacy, 1);

#--------------------------------
# Internal Variables. Do not monkey with.
$CLat = NULL;
$CLon = NULL;
$GmapE = NULL;
$GmapIEFix = '';
$GmapJS = ' ';
$GmaMarkers = array();
$GmapN = NULL;
$GmaPoints = array();
$GmaPolyLines = array();
$GmaPolylines = array();
$GmapS = NULL;
$GmapW = NULL;

Markup('gmap', 'block', '/\(:gmap(.*?):\)/e', 'Gmap("$1");');
Markup('gmapoint','_begin','/\(:gmapoint(.*?):\)/e','GmaPoint("$1");');
Markup('gmaicon','_begin','/\(:gmaicon(.*?):\)/e','GmaPoint("$1",1);');
Markup('gmapolyline','_begin','/\(:gmapolyline(.*?):\)/e','GmaPolyLine("$1");');

require_once('gmap_api/gma_icon.class.php');
require_once('gmap_api/gma_point.class.php');
require_once('gmap_api/gma_polyline.class.php');
function GmaZip($z) {
    global $GmapZIPFile;
    $z = escapeshellcmd($z);
    if ($GmapZIPFile && file_exists($GmapZIPFile)){
      exec("grep '^$z' $GmapZIPFile", $r);
      list($lat, $lon) = split(':', $r[0], 2);
      return array($lat, $lon);
    }
    return array(0,0); # No zip file.
}
function GmaPolyLine($i) {
    global $GmaPolylines;

    GmaSpoofLocale();
    $opts = ParseArgs($i);

    #================================
    # Match this point with its line.
    $lid = "polyline".GmapDefaults($opts['id'], '0');
    if (!array_key_exists($lid, $GmaPolylines)) {
        $GmaPolylines[$lid] = new GmaPolyline;
    }
    $line = &$GmaPolylines[$lid];
    #--------------------------------

    if ($opts['zip']) list($opts['lat'], $opts['lon']) = GmaZip($opts['zip']);
    if ($opts['thick']) $line->thickness = $opts['thick'];
    if ($opts['color']) $line->color = $opts['color'];
    $line->addPoint($opts['lat'], $opts['lon']);

	#setlocale(LC_NUMERIC,$_locale);
    GmaUnspoofLocale();

    return '';
}
function GmaIcon($o) {
    # Get Icon parameters.
    $icon = new GmaIcon;
    if ($o['mletter']) $icon->mletter = $o['mletter'];
    if ($o['mm']) $icon->mm = $o['mm'];
    if ($o['pal']) $icon->pal = $o['pal'];
    if ($o['icon']) $icon->inum = $o['icon'];
    #print_r($o);
    # Create Icon Object
    # return object.
    return $icon;
}
function GmaPoint($i, $has_icon=0) {
    global $GmaMarkers, $GmaPoints, $GmapZIPFile;

    GmaSpoofLocale();

    $opts = ParseArgs($i);
    if ($opts['zip']) list($opts['lat'], $opts['lon']) = GmaZip($opts['zip']);
    if (!$opts['lat'] or !$opts['lon']) return '';

    $point = new GmaPoint($opts['lat'], $opts['lon']);
    $point->message = GmapDefaults($opts['msg'], '');
    $point->directions = GmapDefaults($opts['daddr'], $GmapDaddr);
    if ($has_icon) {
      $point->icon = ($has_icon) ? GmaIcon($opts) : '';
    }
    if ($opts['link']) {
      $point->addLink($opts['link']);
    }
    array_push($GmaPoints, $point);
    GmaUnspoofLocale();
    return Keep($point->link());
}

function Gmap($input='') {
    global $EnableGmapTypeControl, $EnableGmapOverviewControl;
    global $EnableGmapCenterAsPoint, $EnableGmapCalculatedCenter;
    global $GmapLon, $GmapLat, $GmapZom, $GmapJS;
    global $GmapCtrl, $GmapKey, $GmaPoints, $GmapType;
    global $GmapHeight, $GmapWidth;
    global $HTMLHeaderFmt, $HTMLStylesFmt;
    global $MapHtml, $ScriptHtml;
    global $GmapN, $GmapS, $GmapE, $GmapW, $CLat, $CLon;
    global $GmaPolyLines, $GmaPolylines;
    global $_GmapPLColor, $_GmapPLThick, $_GmapPLOpacy;
    global $GmapPLThick;

	#$_locale = setlocale(LC_NUMERIC,'0');
	#setlocale(LC_NUMERIC,'en_US');
    GmaSpoofLocale();

    $opt = ParseArgs($input);
    if ($opt['zip']) list($opt['lat'], $opt['lon']) = GmaZip($opt['zip']);
    $zom = ($opt['zom']) ? $opt['zom'] : $GmapZom;
    $lat = ($opt['lat']) ? $opt['lat'] : $GmapLat;
    $lon = ($opt['lon']) ? $opt['lon'] : $GmapLon;

    $cpoint = GmapToggle($EnableGmapCenterAsPoint, $opt['cpoint'], 1);
    $ccalc  = GmapToggle($EnableGmapCalculatedCenter, $opt['ccalc'], 1);

    #--------------------------------
    # Overview - We add if it's in the default or author requests.
    $overview = GmapToggle($EnableGmapOverviewControl, $opt['oview'],
        'map.addControl(new GOverviewMapControl());');

    #--------------------------------
    # Control - We add if it's in the default or author requests.
    $mt_ctrl = GmapToggle($EnableGmapTypeControl, $opt['mtype'],
        'map.addControl(new GMapTypeControl());');

    #--------------------------------
    # Get the Controller Size. v1.0.1 (v.1.0.2 bugfixes by Des
    $cki = ($opt['ctrl']) ? $opt['ctrl'] : $GmapCtrl;
	$map_controller_size = (strtolower($cki) == 'large')
		? "map.addControl(new GLargeMapControl());"
		: "map.addControl(new GSmallMapControl());";

    #--------------------------------
    # Get the Map type. v1.0.1 (inserted in Gmap2.setCenter())
    $map_type = array(
        'hyb' => 'G_HYBRID_MAP',
        'nor' => 'G_NORMAL_MAP',
        'sat' => 'G_SATELLITE_MAP',
    );
    $map_type_key = GmapDefaults($opt['type'], $GmapType);
    #$map_type_key = ($opt['type']) ? $opt['type'] : $GmapType;
    $map_type = $map_type[strtolower(substr($map_type_key,0,3))];

    #================================
    # Let's the author change the default size of the map.
    #--------------------------------
    $height = GmapDefaults($opt['height'], $GmapHeight);
    $width = GmapDefaults($opt['width'], $GmapWidth);

    if ($lat != '' and $lon != '') {
          $point = new GmaPoint($lat, $lon);
          $point->message = $opts['msg'];
          $point->directions = $opts['daddr'];
          array_push($GmaPoints, $point);
    }

    #--------------------------------
    # Build the various points.
    $c = 0;
    $markers = ''; 
    $points = ''; 
    $polylines = ''; 
    $overlays = ''; 
    $windows = ''; 
    $outputs = '';
    foreach ($GmaPoints as $point) {
        #$markers .= $point->marker();
        #$outputs .= $point->output();
        #$overlays .= $point->overlay();
        $points .= $point->pointer();
        #$windows .= $point->window();
        #if ($point->icon) {
        #  $icon = $point->icon;
        #  $icons .= $icon->icon();
        #}
    }
    if (count($GmaPolylines)) {
        foreach($GmaPolylines as $id => $line) {
          $overlays .= $line->overlay();
          $polylines .= $line->polyline();
        }
    }

    if ($opt['float']) $float = "float: $opt[float]";
    #--------------------------------
    # Set the HTML
    $HTMLStylesFmt['gmap_api'] = "div#map{ height: $height; width:$width;}";
    $HTMLHeaderFmt[]
        = '<style type="text/css">v:* { behavior:url(#default#VML); }</style>'
		  .'<script src="http://maps.google.com/maps?file=api&v=2&key='
          .$GmapKey.'" type="text/javascript"></script>';
    $HTMLHeaderFmt[] = "<script language='javascript' src='\$PubDirUrl/scripts/gmaJs.js'></script>";

    $center = ($cclac == 1)
       ? "var center = new GLatLng($CLat,$CLon);\n"
       : "var center = new GLatLng($lat,$lon);\n";
$GmapJS =<<<GMAPJS
<script type="text/javascript">
  //<![CDATA[
    // Copyright (c) 2006, Benjamin C. Wilson. All Rights Reserved.
    // Google Map API for PmWiki.
    // This copyright statement must accompany this script.

    if (GBrowserIsCompatible()) {
      var from_htmls = [];
      var htmls = [];
      var markers = [];
      var to_htmls = [];
      var i = 0;
      var map = new GMap2(document.getElementById('map'));
      var bounds = new GLatLngBounds;
$points
$icons
$polylines
      var level = map.getBoundsZoomLevel(bounds);
      var clat = (bounds.getNorthEast().lat() + bounds.getSouthWest().lat()) /2;
      var clng = (bounds.getNorthEast().lng() + bounds.getSouthWest().lng()) /2;
      map.setCenter(new GLatLng(clat,clng), level, $map_type);
      map.addControl(new GScaleControl()); // v1.0.1 - by Des
      $mt_ctrl
      $map_controller_size // v1.0.1 - by Ben Wilson
      $overview
      doOverlay(map);

$overlays
$windows
    }
    else {
      alert("Sorry, the Google Maps API is not compatible with this"
      + "browser. Please visit http://www.mapquest.com for an alternative.");
    }
  //]]>
</script>
GMAPJS;

	#setlocale(LC_NUMERIC,$_locale);
    GmaUnspoofLocale();

    return "$MapHtml";
}
function GmapDefaults($option, $default) {
    return ($option) ? $option : $default;
}
function GmapToggle($d, $o, $t) {
    #- $o = y or n, overrides site default
    if ($o == 'n') return '';
    if ($o == 'y') return $t;
    return ($d) ? $t : '';
}

function GmaSpoofLocale($off=0) {
    global $GmapHoldLocale;
    if ($off) {
	  setlocale(LC_NUMERIC,$GmapHoldLocale);
    }
    else {
      $GmapHoldLocale = setlocale(LC_NUMERIC,'0');
	  setlocale(LC_NUMERIC,'en_US');
    }
}
function GmaUnspoofLocale() {
    GmaSpoofLocale(1);
}
function GmaDoIEFix() {
  global $GmapIEFix, $FarmC;
  if (get_cfg_var('browscap')) {
   $browser=get_browser(); //If available, use PHP native function
  }
  else {
   require_once('gmap_api/php-local-browscap.php');
   $browser=get_browser_local(null, false, "$FarmC/gmap_api/browscap.ini");
  }
  $GmapIEFix = (preg_match('/IE/', $browser->browser))
    ? " xmlns='http://www.w3.org/1999/xhtml' xmlns:v='urn:schemas-microsoft-com:vml'"
    : '';
}
